<?php defined('_JEXEC') or die('Restricted access');

/**
 * @version 2.0 $Id: controller.php,v 1.19 2009/12/02 20:38:12 harry Exp $
 * @package Joomla
 * @subpackage H2N Link Directory 2
 * @copyright (C) 2008 - 2009 Harry Klein - www.joomla-hklein.de
 * @Copyright (C) 2006 Soner (pisdoktor) Ekici - www.sonerekici.com
 * @license GNU/GPL, see LICENSE.php
 * 
 * H2N Link Directory 2 is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 2
 * as published by the Free Software Foundation.
 *
 * H2N Link Directory 2 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EventList; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

jimport('joomla.application.component.controller');

require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'LDConfig.php');
require_once(JPATH_COMPONENT.DS.'classes'.DS.'LinkdirectoryURL.php');
require_once JPATH_COMPONENT.DS.'models'.DS.'comment.php';
require_once JPATH_COMPONENT.DS.'models'.DS.'log.php';

class LinkDirectory2Controller extends JController {

	function display() {
		parent::display();
	}

	/**
	 * Liefert den Link mit der Id, die aus dem Request geleden worden ist.
	 * Bei einem Fehler wird redirect entsprechend gesetzt
	 *
	 * @return Link mit der id aus dem Rquest
	 * @throws Exception, wenn die Zeile mit der Id des Request nicht
	 *         gefunden worden ist.
	 */
	private function &loadLinkRow(){
		$db = JFactory::getDBO();
		JTable::addIncludePath(JPATH_COMPONENT_ADMINISTRATOR . DS . 'tables');
		$row =& JTable::getInstance('weblink', 'Table');
		$id = JRequest::getInt('id', 0);
		$row->load($id);

		if ($row === false){
			$this->setRedirect(LinkdirectoryURL::getCatURL('categories',null), 'INVALID LINK OR LINK IS NOT PUBLISHED' );
			throw new Exception("Can't find Link with ID [$id]");
		}
		return $row;
	}

	/**
	 * Überprüfung, ob das Rating-Form angezeigt werden darf. 
	 *
	 */
	public function rate(){
		$ld = new LDConfig();
		try {
			$row = &$this->loadLinkRow();
			$serviceEnable = $ld->ratelink;
			$needLogin = ! $ld->gaddrate;
			$onlyOnce = $ld->rateonce;
			if ($this->checkLog($row,$serviceEnable,$needLogin,$onlyOnce,'vote')){
				$this->display();
			}
		} catch (JException $e) {
			return;
		}
	}

	
	/**
	 * Überprüfung, ob das interne Kommentar-Form angezeigt werden darf. 
	 *
	 */
	public function comment(){
		$ld = new LDConfig();
		try {
			$row = &$this->loadLinkRow();
			$serviceEnable = $ld->commentlink && ($ld->commentSystem =='intern');
			$needLogin = ! $ld->gaddcom;
			$onlyOnce = $ld->writeonce;
			if ($this->checkLog($row,$serviceEnable,$needLogin,$onlyOnce,'comment')){
				$this->display();
			}
		} catch (JException $e) {
			return;
		}
	}

	/**
	 * Überprüft, ob die Aktion  $action erlaubt ist.
	 *
	 * @param int $row Link selbst
	 * @param boolean $serviceEnable true, wenn Service Enable ist
	 * @param boolean $needLogin true, wenn Login erforderlich ist
	 * @param boolean $onlyOnce true, wenn pro User die Aktion nur einmal erlaubt ist.
	 * @param String $action Aktion, z.Z. vote und comment erlaubt
	 * @return true, wenn AKtion erlaubt ist, sonst false und entsprechend gesetztem Redirect
	 */
	private function checkLog(&$row,$serviceEnable,$needLogin,$onlyOnce,$action){
		# User IP Address und User beschaffen
		$vote_ip = getenv( 'REMOTE_ADDR' );
		$user = &JFactory::getUser();

		// TODO Test, ob Categorie veröffentlich ist, fehlt noch
		if (empty($row)|| $row->published != 1) {
			// Link does not exists, or is not published
			$this->setRedirect(JRoute::_(LinkdirectoryURL::getCatURL('categories',null)), JText::_('INVALID LINK OR LINK IS NOT PUBLISHED' ));
			return false;
		}

		if ( $serviceEnable == false ) {
			# Service not enable
			$this->setRedirect(JRoute::_(LinkdirectoryURL::getItemUrl('link',null,$row->id)), JText::_('SERVICE NOT ENABLE') );
			return false;
		}

		if ( ($needLogin == true) && ($user->id < 1)) {
			# User is not logged in
			$this->setRedirect(JRoute::_(LinkdirectoryURL::getItemUrl('link',null,$row->id)), JText::_('LD_LOGIN_FIRST') );
			return false;
		}

		if ( !$onlyOnce){
			return true;
		}

		$logModel = new LinkDirectory2ModelLog($action);
		$voted = $logModel->getHasUserLogEntryForLink($row->id);
		if ( !$voted ) {
			return true;
		}
		switch ($action){
			case 'vote':
				$message=JText::_('LD RATE BEFORE'); 
				break;
			case 'comment':
				$message=JText::_('LD_COMMENT_BEFORE'); 
				break;
			default:
				$message=JText::_('Internal error');
		}
		$this->setRedirect(JRoute::_(LinkdirectoryURL::getItemUrl('link',null,$row->id)), $message );
		return false;
	}

	public function addRate(){
		$rating = JRequest::getInt( 'rating', 0 );

		try {
			$row = &$this->loadLinkRow();
			$ld = new LDConfig();
			$serviceEnable = $ld->ratelink;
			$needLogin = ! $ld->gaddrate;
			$onlyOnce = $ld->rateonce;
			if (! $this->checkLog($row,$serviceEnable,$needLogin,$onlyOnce,'vote')){
				return;
			}

			if ( $rating == 0 ) {
				$this->setRedirect(JRoute::_(LinkdirectoryURL::getItemUrl('link',null,$row->id)), 'LD_CHOOSE_RATE' );
				return;
			}

			if ( $ld->rateonce) {
				$logModel = new LinkDirectory2ModelLog($action);
				$logModel->updateLog($row->id,'vote');
			}

			$model = $this->getModel('link');

			if ($model->updateRating($row->id,$rating)) {
				$msg = JText::_( 'LD RATE SUCCESS' );
			} else {
				$msg = JText::_( 'LD RATE FAILED' );
			}
			$this->setRedirect(JRoute::_(LinkdirectoryURL::getItemUrl('link',null,$row->id)), $msg);
		} catch (Exception $e){
			return;
		}
	}

	/**
	 * Fügt ein neurn Kommentar hinzu
	 *
	 */
	public function addComment(){
		global $mainframe;
		require_once JPATH_COMPONENT.DS.'classes'.DS.'CaptchaUtils.php';
		$reloadPage = false;
		$checkSecurity = CaptchaUtils::validateCaptcha();
		if (!$checkSecurity) {
			$mainframe->enqueueMessage(JText::_('LD_LINK_REFUSED'));
			$reloadPage = true;
		}

		$post	= JRequest::get('post');
		$row = & JTable::getInstance('ldcomment','Table');
		if (!$row->bind(JRequest::get('post'))) {
			JError::raiseError( 500, $db->stderr() );
			return false;
		}

		switch ($row->type){
			case 1:
			case 2:
			case 3:
			case 4:
				$row->rev_title=JText::_('LD_R'.$row->type);
				break;
			case 0:
			default:
				break;
		}
		
		if (!$row->check()){
			if ($row->getError());
			$mainframe->enqueueMessage($row->getError() == '' ? JText::_('LD_MISSING_A_FIELD') : $row->getError());
			$reloadPage = true;
		}

		if ($reloadPage){
			JRequest::setVar( 'view'  , 'link' );
			switch ($row->type){
				case 1:
				case 2:
				case 3:
				case 4:
					JRequest::setVar( 'task'  , 'report' );
					break;
				case 0:
				default:
					JRequest::setVar( 'task'  , 'comment' );
					break;;
			}
			parent::display();
			return;
		}

		$model = new LinkDirectory2ModelLdcomment();

		if ($model->store($row)) {
			$msg = JText::_( 'Comment Saved' );
		} else {
			$msg = JText::_( 'Error Saving Comment' );
		}
		$this->setRedirect(JRoute::_(LinkdirectoryURL::getItemUrl('link',null,$row->link_id)), $msg );
	}
}
?>
